#include <stdio.h>
#include <stdbool.h>
#include <string.h>

#include <logs_text3.h>
#include <log_it.h>

/*
 * It is more or less "grep -v string file >to_file"
 *  
 *  Return:
 *  On sucess: 	number of lines that are removed now
 *  Error:
 *  	-1	No file string
 *  	-2	No target file string
 *  	-3	No string to search
 *  	-4	Cannot open file
 *  	-5	Cannot open target file
 *  	-6	Cannot open logfile
 */

int remove_lines(const char *file,
                const char *to_file,
		          const char *string,
		          int verbose, 
                const char *logfile, 
                const char *key)
{
   FILE *f_read, *f_write, *f_log;
   char buffer1[BUFFER + 1] = { '\0' };
   char  *grep;
   int lines_removed = 0;
   int lines_written = 0;
   int lines_org = 0;
   bool found = false;
   
   /* Idiot checking (newer trust programmers!) */
   if(!file) {
	   fprintf(stderr, "[ERROR] No file to clean!\n");
	   return -1;
   }
   if(!to_file) {
	   fprintf(stderr, "[ERROR] No file to write to!\n");
	   return -2;
   }
   if(!string) {
	   fprintf(stderr, "[ERROR] No string for cleaning!\n");
	   return -3;
   }

   /* give status message */
   if(verbose > 2) {
	   printf("[STATUS]	grep -v %s in %s\n", string, file);
   }
   
   /* Open all files */
   if((f_read = fopen(file, "r")) == 0) {
	   fprintf(stderr, "[ERROR] in read opening %s\n", file);
	   perror("[ERROR] replace_in_file(): ");
	   return -4;
   }
   if((f_write = fopen(to_file, "w")) == 0) {
	   fprintf(stderr, "[ERROR] in write opening %s\n", to_file);
	   perror("[ERROR] replace_in_file(): ");
	   return -5;
   }
   if(logfile) {
	   if((f_log = fopen(logfile, "w")) == 0) {
	      fprintf(stderr, "[ERROR] in write opening %s\n", logfile);
	      perror("[ERROR] replace_in_file(): ");
	      return -6;
	   }
   }
   
   
   /* Ok let's start */
   while (!feof(f_read)) {	/* reading file */
	   /* reset */
	   memset(buffer1, 0, BUFFER);
	   fgets(buffer1, BUFFER, f_read);
	   lines_org++;
	   grep = 0;
	   
      /* this is not a logentry, so ignore it */
	   if(strlen(buffer1) < 1) {
	      break;
	   }
	   
   	/* grep for string */
	   grep = strstr(buffer1, string);
	   if(grep != 0) {	/* found */
	      found = true;
	   }
      if(found == true) {	/* it matches string */
         if(verbose > 4) {
            printf("[FOUND]\t %s", buffer1);
         }
         if(logfile) {
            if(key) {
               crypt_log_it(logfile, key, "[FOUND]\t %s",buffer1);
            } else{
               log_it(logfile, "[FOUND]\t %s",buffer1);
            }
         }
      } else {	/* entry is clean, so it can be written to file */
         fprintf(f_write, "%s",buffer1);
         lines_written++;
      }

   }	/* end reading file */

   fclose(f_write);
   fclose(f_read);

    /* calculate statistic */
   lines_org--;
   lines_written--;

   lines_removed = lines_org - lines_written;
      
   if(verbose > 2) {
      printf("[STATUS] From %d lines %d lines written -> %d lines removed\n",
              lines_org, lines_written, lines_removed);
   }
    
   return(lines_removed);
}
