#ifndef linux
#define ut_user ut_name
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stdbool.h>
#include <string.h>

#ifdef freebsd 
#include <sys/types.h>
#endif 

#include <utmp.h>

/* hmm, networking in a logcleaner... a backdoor?
 * no, just lazy, and use inet_aton()
 * ...but read the code, if you don't trust...
 */
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <log_it.h>
#include <logs_wtmp.h>


/*
 * wtmpLogs() removes/changes logs in utmp/wtmp logfiles
 *   Don't forget other logfiles (lastlog), so that eveything looks Ok
 *   
 * 	file 		the utmp/wtmp logfile
 * 	to_file		write into this file
 * 	string		generic string to look for (username/host/ip)
 * 	replace		replace with this string
 * 	user		clean this user
 * 	host		clean this host
 *
 * 	Return:
 * 	  Sucess:	number of changes/deleted entrys
 * 	  Error:
 * 	  		-1	no utmp/wtmp file
 * 	  		-2	no file to write
 * 	  		-3 	nothing to clean (string/user/host)
 * 	  		-4	error in opening file to read
 * 	  		-5	error in opening to_file to write
 * 	  		
 * 	  		
 */
int wtmpLogs(const char *file,
             const char *to_file,
	          const char *string,
	          const char *replace,
	          const char *user,
	          const char *host,
	          int verbose, 
             const char *logfile, 
             const char *key)
{
   FILE *f_read, *f_write;
   bool found1 = true, found2 = false;
   bool found_host = false, found_user = false, saved = false;
   struct utmp info;
   struct utmp save;
   int total = 0, cleared = 0;
   char *check = 0;
   char ipadr[255 + 1] = { '\0' };
   struct in_addr bla;

   /* Idiot checking (newer trust programmers!) */
   if(!file) {
	   fprintf(stderr, "[ERROR] No file to clean!\n");
	   return -1;
   }
   if(!to_file) {
	   fprintf(stderr, "[ERROR] No file to write to!\n");
	   return -2;
   }
   if((!string) && (!user) && (!host)) {
	   fprintf(stderr, "[ERROR] No string for cleaning!\n");
	   return -3;
   }

   /* give status message */
   if(verbose > 2) {
	   printf("[STATUS]	Clear wtmp/utmp in %s\n", file);
   }
   
   /* Open all files */
   if((f_read = fopen(file, "r")) == 0) {
	   fprintf(stderr, "[ERROR] in read opening %s\n", file);
	   perror("[ERROR] replace_in_file(): ");
	   return -4;
   }
   if((f_write = fopen(to_file, "w")) == 0) {
	   fprintf(stderr, "[ERROR] in write opening %s\n", to_file);
	   perror("[ERROR] replace_in_file(): ");
	   return -5;
   }

   /* start reading file */
   while(!feof(f_read)) {
	   /* reset */
	   memset(&info, 0, sizeof(info));
	   memset(ipadr, 0, 255);
	   found1 = false;
	   found2 = false;
	   total++;
	   
      if(fread(&info, sizeof(info), 1, f_read)) {
	      /* ut_host */
	      if(host) {
		      /* grep */
	      	check = 0;
		      check = strstr(info.ut_host, host);
		      if(check) { /* string found */
		         found_host = true;
		         if(saved == false) {
			         /* save entry for verbose info/logging */
			         memcpy(&save, &info, sizeof(struct utmp));
			         saved = true;
		         }
		         if(replace) { /* replace it with $replace */
			         memset(&info.ut_host, 0, UT_HOSTSIZE);
			         strncpy(info.ut_host, replace, UT_HOSTSIZE - 1);
			         found_host = false;
			         found2 = true;
		         }
		      }
	      } else{		/* check generic string */
		      if(string) {
		         check = 0;
		         check = strstr(info.ut_host, string);
		         if(check) {
			         found_host = true;
			         if(saved == false) {
			            memcpy(&save, &info, sizeof(struct utmp));
			            saved = true;
			         }
			         if(replace) {
			            memset(&info.ut_host, 0, UT_HOSTSIZE);
			            strncpy(info.ut_host, replace,
				         UT_HOSTSIZE - 1);
			            found_host = false;
			            found2 = true;
			         }
		         }
		      }
	      }
#ifdef linux         
	      /* ut_addr */
	      /* make to ut_addr to string */
	      snprintf(ipadr, 255, "%d.%d.%d.%d",
                  (info.ut_addr) & 0xFF,
		            (info.ut_addr >> 8) & 0xFF,
		            (info.ut_addr >> 16) & 0xFF,
		            (info.ut_addr >> 24) & 0xFF);

	      if(host) {
		      check = 0;
		      check = strstr(ipadr, host);
		      if(check) {
		         found_host = true;
		         if(saved == false) {
			         memcpy(&save, &info, sizeof(struct utmp));
			         saved = true;
		         }
		         if(replace) {
			         if(inet_aton(replace, &bla) == 0) {
			            fprintf(stderr,"[ERROR] \t %s seems NOT to be an\
                             IP-Adress!\n", replace);
			         }
		   	      memset(&info.ut_addr, 0, sizeof(struct in_addr));
			         memcpy(&info.ut_addr, &bla.s_addr,
			         sizeof(struct in_addr));
			         found_host = false;
			         found2 = true;
		         }
		      }
	      } else {
		      if(string) {
		         check = 0;
		         check = strstr(ipadr, string);
		         if(check) {
			         found_host = true;
			      if(saved == false) {
			         memcpy(&save, &info, sizeof(struct utmp));
			         saved = true;
			      }
			      if(replace) {
			         if(inet_aton(replace, &bla) == 0) {
				         fprintf(stderr,"[ERROR] \t %s seems NOT to be an\
                             IP-Adress!\n", replace);
			         }
			         memset(&info.ut_addr, 0,
				      sizeof(struct in_addr));
			         memcpy(&info.ut_addr, &bla.s_addr,
				      sizeof(struct in_addr));
			         found_host = false;
			         found2 = true;
			      }
		      }
		   }
	    }
#endif

       /* ut_user */
	    if(user) {
          check = 0;
	   	 check = strstr(info.ut_user, user);
		    if(check) {
             found_user = true;
		       if(saved == false) {
                memcpy(&save, &info, sizeof(struct utmp));
			       saved = true;
		       }
		       if(replace) {
			       memset(&info.ut_user, 0, UT_NAMESIZE);
			       strncpy(info.ut_user, replace, UT_NAMESIZE - 1);
			       found_user = false;
			       found2 = true;
		       }
		    }
	    } else {
          if(string) {
             check = 0;
		       check = strstr(info.ut_user, string);
		       if(check) {
                found_user = true;
			       if(saved == false) {
			          memcpy(&save, &info, sizeof(struct utmp));
			          saved = true;
			       }
			    if(replace) {
			       memset(&info.ut_user, 0, UT_NAMESIZE);
			       strncpy(info.ut_user, replace,
				    UT_NAMESIZE - 1);
			       found_user = false;
			       found2 = true;
			    }
		    }
		 }
	 }
    /* end of checks */

	 /* now: check if matches to all */
	 if((((user) && (found_user == true))
        && ((host) && (found_host == true))) || (((user)
		  && (found_user == true))
		  && (!host))
		  || ((!user) && ((host) && (found_host == true)))
		  || (((!user) && (!host)) && (found_host == true))
		  || (((!user) && (!host)) && (found_user == true))
        )
	    found1 = true;
	 else
       found1 = false;

	    /* It is clean, so write it into file */
	    if(found1 == false) {
		    if(fwrite(&info, 1, sizeof(info), f_write) != sizeof(info)) {
             fprintf(stderr, "[ERROR] clearWtmp: write to %s\n", to_file);
		       return -9;
		    }
	    }
	    
       /* verbose info  and logging */
	    if((found1 == true) || (found2 == true)) {	/* deleted or changed */
		    cleared++;
		    /* verbose */
		    if(verbose > 3) {
#ifdef linux            
		      printf("[FOUND] %s  %s %s %s %s %s \n", ipadr, save.ut_host, 
                   save.ut_user, save.ut_id, save.ut_line, save.__unused);
#else
             printf("[FOUND] %s  %s %s \n", save.ut_host,
                    save.ut_user,  save.ut_line);
#endif             
		    }
		    
          /* log it */
		   if(logfile) {
		      if(key) {
#ifdef linux               
			      crypt_log_it(logfile, key, "[FOUND]	%s  %s %s %s %s %s \n",
                            ipadr, save.ut_host, save.ut_user,
				                save.ut_id, save.ut_line,
				                save.__unused);
#else
               crypt_log_it(logfile, key, "[FOUND] %s  %s %s \n", 
                            save.ut_host, save.ut_user,  save.ut_line);
#endif               
		      } else {
 #ifdef linux               
			      log_it(logfile, "[FOUND]	%s  %s %s %s %s %s \n",
                      ipadr, save.ut_host, save.ut_user,
			             save.ut_id, save.ut_line, save.__unused);
#else        
               log_it(logfile, "[FOUND]   %s  %s %s \n",
                      save.ut_host, save.ut_user,  save.ut_line);
#endif
		      }
		   }
	    }
	  }
   } /* end reading file */

   /* Closing files again */
   fclose(f_write);
   fclose(f_read);


   /* be verbose */
   if(verbose > 2) {
	   printf("[STATUS]	cleared %d  of %d log entrys\n", cleared, total);
   }
   
   return(cleared);
}
