#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <utime.h>
#include <unistd.h>
#define __USE_BSD 1
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <utils_touch.h>

/* converts from "YYYY:MM:DD:hh:mm:ss:uuuuuu" to struct timeval */
/* void char2time(const char *s, time_t time_result) { */
time_t char2time(const char *s)
{
   time_t time_result;
   struct timeval tvp;
   struct tm tm;
#define DELIM   ':'


   memset(&tm, 0, sizeof(tm));
   if(s) {
	   tm.tm_year = atoi(s) - 1900;
	   if((s = strchr(s, DELIM))) {
	      tm.tm_mon = atoi(++s) - 1;
	      if((s = strchr(s, DELIM))) {
		      tm.tm_mday = atoi(++s);
		      if((s = strchr(s, DELIM))) {
		         tm.tm_hour = atoi(++s);
		         if((s = strchr(s, DELIM))) {
			         tm.tm_min = atoi(++s);
			         if((s = strchr(s, DELIM))) {
			            tm.tm_sec = atoi(++s);
			            if((s = strchr(s, DELIM))) {
				            tvp.tv_usec = atoi(++s);
			            }
			         }
		         }
		      }
	      }
	   }
   }
   time_result = mktime(&tm);
   
   return(time_result);
}

/* taken from touch2 
 *	changes ctime of a file (last-inode-change)
 *	by chaning systime
 *		 change_ctime(const char *file, const struct timeval *ctime, struct stat inode) 
 *		 RETURN: 
 *			 0: everything is fine
 *		  	-1: error in gettimeofday()
 *			-2: error in settimeofday()
 *			-3: error in utimes()
 *			-4: errro in settimeofday() for the system
 */
int change_ctime(const char *file, const struct timeval *ctime, 
                 struct stat *inode)
{
   struct timeval tv[2], now;

   memset(&tv, 0, sizeof(tv));
   /* st_[am]time may be either struct timespec or time_t
    * the first member of struct timespec is tv_sec
    * (the same as struct timeval)
    */
   memcpy(&tv[0], &inode->st_atime, sizeof(inode->st_atime));
   tv[0].tv_usec /= 1000;	/* nanosecs to microsecs */
   memcpy(&tv[1], &inode->st_mtime, sizeof(inode->st_mtime));
   tv[1].tv_usec /= 1000;	/* nanosecs to microsecs */

   /* Save current time */
   if(gettimeofday(&now, 0) < 0) {
	   fprintf(stderr, "[ERROR] gettimeoftoday!");
	   return -1;
   }

   if(settimeofday(ctime, NULL) < 0) {
	   fprintf(stderr, "[ERROR] settimeoftoday!");
	   return -2;
   }

   if(utimes(file, tv) < 0) {
	   fprintf(stderr, "[ERROR] utimes!");
	   return -3;
   }

   /* Restore system time */
   if(settimeofday(&now, 0) < 0) {
	   fprintf(stderr, "[ERROR] settimeoftoday!");
	   return -4;
   }

   return 0;
}

/*
 * simple funktion to change MAC time
 * executing change_ctime() to change ctime
 * RETURN:
 *	   1	not root -> not allowed to change ctime (change only atime and mtime)
 *		0	fine no problems
 *		-1	error in gettimeofday()
 *		-2	error in change_ctime()
 *		
 */
int set_time(const char *file, struct stat *perm)
{
   struct utimbuf buf;
   struct timeval new_ctime;	/* New inode-change time */
   
   buf.actime = perm->st_atime;
   buf.modtime = perm->st_mtime;

   if(utime(file, &buf) != 0) {
	   printf("[WARN] error in utime() %s \n", file);
   }
   /* You can change ctime only as root. atime and mtime can be changed as user */

   if(getuid()) {
	   printf("[WARN]\t sorry cannot change ctime: you are not root! \n");
	   return 1;
   }

   if(gettimeofday(&new_ctime, 0) < 0) {
	   fprintf(stderr, "[ERROR] gettimeoftoday!");
	   return -1;
   }
   memset(&new_ctime, 0, sizeof(new_ctime));
   
   /* st_ctime may be either struct timespec or time_t
    * the first member of struct timespec is tv_sec
    * (the same as struct timeval)
   */
   memcpy(&new_ctime, &perm->st_ctime, sizeof(time_t));
   new_ctime.tv_usec /= 1000;	/* nanosecs to microsecs */

   if(change_ctime(file, &new_ctime, perm) < 0) {
	   fprintf(stderr, "[ERROR] change ctime\n");
	   return -2;
   }
   
   return 0;
}

/*
 * simple touc with c-time modifikation
 * 	RETURN:  0  = ok
 * 		>0  = not root, cannot change c-time
 * 		<0  = error 
 */
int touch3(const char *orgfile, const char *changefile)
{
   struct stat perm;
   int ret;

   if(stat(orgfile, &perm) != 0) {	/* file exits? */
	   printf("[IDIOT] %s does NOT exist! \n", orgfile);
	   return -1;
   }
   ret = set_time(changefile, &perm);

   return(ret);
}
